﻿
/// <summary>
/// コマンド書式を定義する。
/// 
/// clsDataFormatPallApp は AppTwelite などで利用される以下のような形式となる。ペイロードバイト列は IDataPayload により規定する。
/// 4: [中継器のシリアル番号]
/// 1: [LQI値]
/// 2: [続番号]
/// 4: [送信元のシリアル番号]
/// 1: [送信元の論理ID]
/// 1: [0x80 固定 これ以外は AppTagでの識別子(未実装)]
/// ---  以下のデータは IDataPayload により実装
/// 1: [PAL基板識別子]
/// 1: [センサデータ数]
/// 4: [センサーデータヘッダ]
/// N: [センサーデータ]
/// 
/// </summary>
/// <remarks></remarks>
using System.Diagnostics;
using System.Linq;
using System.Collections.Generic;
using System.Collections;
using System;
using System.Xml.Linq;
using MONOW_PAL_PARSER;

namespace MONOW_PAL_PARSER
{
    public class ClsDataFormatPalApp : ITweDataFormat
    {
        private bool _bStored;
        private ITweDataPayload _dataPayload;

        private UInt32 _uiRepeater;
        private byte _bLQI;
        private UInt16 _usDataSer;
        private UInt32 _uiSrcAddr;
        private byte _bSrcAddr;
        private byte _bPayFmt;
        private byte _bPalPcb;

        /// <summary>
        /// バイト列の解釈と ITweDataPayload の生成
        /// </summary>
        /// <param name="b"></param>
        /// <returns></returns>
        public bool parseByteArray(ref byte[] b)
        {
            bool _parseByteArray = false;
            clearData();

            int i = 0;
            try
            {
                UiRepeater = PAL_Utils.get_BE_DWORD(ref b, i); i += 4;
                BLQI = b[i]; i += 1;
                UsDataSer = PAL_Utils.get_BE_WORD(ref b, i); i += 2;
                UiSrcAddr = PAL_Utils.get_BE_DWORD(ref b, i); i += 4;
                BSrcAddr = b[i]; i += 1;
                BPayFmt = b[i]; i += 1;
                BPalPcb = b[i]; i += 1;

                // should check the last byte (CRC8)
                byte c = PAL_Utils.Calc_CRC8_CCITT(ref b, b.Length - 1);
                if (b[b.Length - 1] != c)
                {
                    throw null;
                }

                byte[] b2 = b.Skip(i).ToArray();
                _dataPayload = PAL_Utils.genDataPayloadPAL(ref b2);
            }
            catch (Exception ex)
            {
                _dataPayload = null;
            }

            if (_dataPayload != null)
            {
                _bStored = true;
                _parseByteArray = true;
            }
            else
                clearData();

            return _parseByteArray;
        }

        public byte[] toByteArray()
        {
            byte[] abyPayload;
            byte[] _toByteArray = null;

            if (_bStored & (_dataPayload != null))
            {
                abyPayload = _dataPayload.ToByteArray();

                if (abyPayload != null)
                {
                }
            }
            return _toByteArray;
        }

        private void clearData()
        {
            _bStored = false;
            _dataPayload = null;
        }

        public ClsDataFormatPalApp()
        {
            clearData();
        }

        public ClsDataFormatPalApp(ref byte[] b)
        {
            parseByteArray(ref b);

            if (!_bStored)
                throw new Exception();
        }

        public byte byFormatType
        {
            get
            {
                return PAL_Utils.twePayloadFmtTypePal;
            }
        }

        public ITweDataPayload objPayload
        {
            get
            {
                return _dataPayload;
            }
        }

        public uint UiRepeater
        {
            get
            {
                return _uiRepeater;
            }
            set
            {
                _uiRepeater = value;
            }
        }

        public byte BLQI
        {
            get
            {
                return _bLQI;
            }
            set
            {
                _bLQI = value;
            }
        }

        public ushort UsDataSer
        {
            get
            {
                return _usDataSer;
            }
            set
            {
                _usDataSer = value;
            }
        }

        public uint UiSrcAddr
        {
            get
            {
                return _uiSrcAddr;
            }
            set
            {
                _uiSrcAddr = value;
            }
        }

        public byte BSrcAddr
        {
            get
            {
                return _bSrcAddr;
            }
            set
            {
                _bSrcAddr = value;
            }
        }

        public byte BPayFmt
        {
            get
            {
                return _bPayFmt;
            }
            set
            {
                _bPayFmt = value;
            }
        }

        public byte BPalPcb
        {
            get
            {
                return _bPalPcb;
            }
            set
            {
                _bPalPcb = value;
            }
        }
    }
}
